/*
 *  isp.c ..... 'C' source for the ISP module, which demonstrates
 *  SLDC In System Programming.
 */


/* -------------------------------------------------------------------------- */
/* HEADER FILES                                                               */
/* -------------------------------------------------------------------------- */

/* The header that defines the Public interface to this module: */
#define DEF_ISP_GLOBALS
#include "isp.h"
#undef DEF_ISP_GLOBALS

/* Additional headers required by this file: */
#include <stdlib.h>
#include <string.h>
#include <Windows.h>
#include <ctype.h>
#include "slcd.h"
#include "crc16.h"


/* -------------------------------------------------------------------------- */
/* PRIVATE CONSTANTS                                                          */
/* -------------------------------------------------------------------------- */

// block erase timeout, in mSec:
#define ispBLOCK_ERASE_TIMEOUT      (1000 * 10)     /* 10 sec */

// upgrade erase timeout, in mSec:
#define ispUPGRADE_ERASE_TIMEOUT    (1000 * 60)     /* 60 sec */

// *CEXT command timeout, in mSec:
#define ispCEXT_CMD_TIMEOUT         (1000 * 15)     /* 15 sec */

// CRC16 is good for 2^16 bits (64K bits) or 8K bytes:
#define ispCHUNK_BYTES              (1024 * 8)      /* 8K */


/* -------------------------------------------------------------------------- */
/* PROTOTYPES FOR PRIVATE CODE                                                */
/* -------------------------------------------------------------------------- */

static BOOL ispEraseExtFlashBlock( SLCD_Model_t model, long blockStartAddrOrSect );


/* -------------------------------------------------------------------------- */
/* PRIVATE DATA                                                               */
/* -------------------------------------------------------------------------- */

// strings indicating boot loader failure message received:
static char *ispUpgradeFailStrings[] =
{
    { "error" },
    { "fail" },
    { "bad" },
    // null terminated list
    NULL,
};


/* -------------------------------------------------------------------------- */
/* PUBLIC CODE                                                                */
/* -------------------------------------------------------------------------- */
// (public types, constants, data, & prototypes in isp.h)

// Validate .BIN file data has correct validation code:
// -- ASSUMES file is already opened for binary read
// -- returns TRUE if successful, else FALSE.
BOOL ISP_ValidateBinFile( SLCD_Model_t model, FILE *fp, long imageFileSize )
{
	BOOL ok = FALSE;
	unsigned short fileValCode;

	// read the 1st 2 bytes of the file (bin data validation code):
	ok = ( fread(&fileValCode, sizeof(unsigned short), 1, fp) == 1 );

	// check validation code is correct for the target model:
	if( ok )
	{
		switch( model )
		{
            case SLCDx:
			case SLCD5:
				ok = ( fileValCode == 0xFEEB );
				break;

			case SLCD5p:
				ok = ( fileValCode == 0xFEEC );
				break;

			default:
				ok = FALSE;
				break;
		}
	}

    // return success/failure:
    return( ok );
}   // ----- end ISP_ValidateBinFile()


// Get SLCDx BMPLoad's trailer data from the end of the file:
// -- only used for SLCDx; SLCD5/SLCD5+ BMPLoad does not append trailer data
// -- ASSUMES file is already opened for binary read
// -- returns TRUE if successful, else FALSE.
BOOL ISP_GetFileTrailerData( FILE *fp, ISP_SLCDx_TrailerData_t *pTrailer )
{
	// unimplemented, and NOT called...
	return FALSE;
}   // ----- end ISP_GetFileTrailerData()


// Get size of External Flash:
// -- returns TRUE if successful, else FALSE.
BOOL ISP_GetExtFlashSize( SLCD_Model_t model, long *flashSize, long *sectorSize )
{
    BOOL ok;
    char *pResp;
    
    // send the command and get the response:
    ok = SLCD_SendCommandStr( "xma" );
    if( ok )
        ok = SLCD_GetResponseStr( &pResp, SLCD_TYPICAL_TIMEOUT );

    // process the response(s):
    if( ok )
    {
        switch( model )
        {
            case SLCDx:
			    ok = ( sscanf(pResp, "%ld", flashSize) == 1 );
                *sectorSize = ( 64 * 1024 );    // 64K
                break;

            case SLCD5:
			case SLCD5p:
			    ok = ( sscanf(pResp, "%ld %ld", flashSize, sectorSize) == 2 );
                break;

            default:
                ok = FALSE;
                break;
        }   // ----- end switch( model )
    }

    // return success/failure:
    return( ok );
}   // ----- end ISP_GetExtFlashSize()


// Erase enough External Flash to hold the image:
// -- returns TRUE if successful, else FALSE.
BOOL ISP_EraseExtFlash( SLCD_Model_t model, long imageSize, long sectorSize )
{
    BOOL ok = TRUE;
    int blkNum = 0;
    int total;
    long addrOrBlock;
    char *str;

    // calculate total number of blocks needed:
	total = ( imageSize / sectorSize );
	if( imageSize % sectorSize )
		total++;

    // start progress indication:
    if( total > 1 )
        str = "blocks";
    else
        str = "block";
    printf( "Erasing %d %s ... 0  ", total, str );

    // do erase 
    for( blkNum = 0; ok && (blkNum < total); blkNum++ )
    {
        addrOrBlock = blkNum;
		// for SLCDx, erase is by address not block, so multiply block (== sector) by sectorSize
        if( model == SLCDx )
            addrOrBlock *= sectorSize;
		ok = ispEraseExtFlashBlock( model, addrOrBlock );
		if( ok )
		    printf( "\b\b\b%-3d", (blkNum + 1) );
	}

    // end progress indication
    printf( "\r\n" );

    // return success/failure:
    return( ok );
}   // ----- end ISP_EraseExtFlash()


// Write the image from the file into External Flash:
// -- ASSUMES file is already opened for binary read
// -- returns TRUE if successful, else FALSE.
BOOL ISP_WriteFileToExtFlash( SLCD_Model_t model, FILE *fp, long imageSize )
{
    BOOL ok = TRUE;
    BOOL firstBuf = TRUE;
    long bytesLeft = imageSize;
    long bytesDone = 0;
    int bytes = 0;
    int percent = 0;
    int lastPercent = 0;
    char *pBuf, *pResp;

	// rewind the file to the beginning:
	rewind( fp );

    // Start write progress indication:
    printf( "Writing %ld bytes ... %%0  ", imageSize );

	// get a buffer big enough to hold the max command line:
	pBuf = SLCD_GetCmdBuf();

	// Tell SLCD to binarily download the whole image into external flash, and to
	// timeout after 2000 mSec of inactivity. Note: "bdld" will send us a '>' prompt
	// when it's ready to receive binary data, or an '!' error prompt if it doesn't
	// like our arguments.
	sprintf( pBuf, "bdld 4 0 %ld 2000", imageSize );
	ok = SLCD_SendCommandStr( pBuf );
    if( ok )
        ok = SLCD_GetResponseStr( &pResp, SLCD_TYPICAL_TIMEOUT );
	if ( ok )
        ok = ( *pResp == '>' );

    // read buffers from the file and send them to the SLCD,
	// relying on the SLCD to throttle data transmission via
	// Xon/Xoff flow control:
    while( ok && (bytesLeft > 0) )
    {
		// determine how many bytes to read into the buffer:
		if( bytesLeft < SLCD_MAXBYTES )
			bytes = bytesLeft;
		else
			bytes = SLCD_MAXBYTES;

		// read that many bytes from the file, into the buffer:
		ok = ( fread(pBuf, sizeof(char), bytes, fp) == bytes );
		if( ! ok )
			break;

		// the 1st buffer is special; we replace the 1st 2 bytes with 0xFF so
		// they remain "erased" until after the whole image is downloaded:
		if( firstBuf )
		{
			pBuf[0] = 0xFF;
			pBuf[1] = 0xFF;
			firstBuf = FALSE;
		}

        // send the buffer to the SLCD:
		ok = SLCD_SendBinaryBuf( pBuf, bytes );

        // update progress and check for bdld failure:
        if( ok )
        {
            bytesLeft -= bytes;
            bytesDone += bytes;
            percent = ((100 * bytesDone) / imageSize );
            if( percent != lastPercent )
            {
                printf( "\b\b\b%-3d", percent );
                lastPercent = percent;
                if( SLCD_GetResponseStr(&pResp, SLCD_MINIMUM_TIMEOUT) )
                    if( *pResp == '!' )
                        ok = FALSE;
            }
        }
    }   // ----- end while() loop

    // end progress indication
    printf( "\r\n" );

	// determine final result:
	// -- it's possible we timed out above before the final response
	//    from "bdld" was available.
	// -- if so, we can definitely expect a response before timing
	//    out here.
	if( ok )
	{
		if( ! strlen(pResp) )
		{
            SLCD_GetResponseStr( &pResp, SLCD_TYPICAL_TIMEOUT );
			ok = ( *pResp == '>' );
		}
	}

    // return success/failure:
    return( ok );
}   // ----- end ISP_WriteFileToExtFlash()


// Validate the External Flash:
// -- ASSUMES file is already opened for binary read
// -- returns TRUE if successful, else FALSE.
BOOL ISP_ValidateExtFlash( SLCD_Model_t model, FILE *fp, long imageSize )
{
    BOOL ok = TRUE;
    BOOL firstBuf = TRUE;
    long address = 0;
    long bytesLeft = imageSize;
    long bytesDone = 0;
    long bytes, bufBytes, chunkBytes;
    unsigned short crcExp;
    unsigned short crcGot;
    int percent = 0;
    int lastPercent = 0;
    char *pBuf, *pResp;

	// get a buffer big enough to hold the max command line:
	pBuf = SLCD_GetCmdBuf();

	// rewind the file to the beginning:
    rewind( fp );

    // Start validation progress indication:
    printf( "Validating %ld bytes ... %%0  ", imageSize );

    // loop on CRC'g the file and the Ext Flash in 8K chunks:
    while( ok && (bytesLeft > 0) )
    {
        // determine how many bytes in this chunk:
        if( bytesLeft < ispCHUNK_BYTES )
            chunkBytes = bytesLeft;
        else
            chunkBytes = ispCHUNK_BYTES;

        // calc a CRC for this chunk, reading a buffer's worth at a time from the file:
        crcExp = SLCD_SEED16;
        for( bytes = chunkBytes; bytes > 0; bytes -= bufBytes )
        {
            // determine how many bytes to read into the buffer:
            if( bytes < SLCD_MAXBYTES )
                bufBytes = bytes;
            else
                bufBytes = SLCD_MAXBYTES;

            // read that many bytes from the file, into the buffer:
            ok = ( fread(pBuf, sizeof(char), bufBytes, fp) == bufBytes );
		    if( ! ok )
			    break;

            // the 1st buffer is special; we replace the 1st 2 bytes with 0xFF since
            // they remain "erased" until after the whole image is validated:
            if( firstBuf )
            {
                pBuf[0] = 0xFF;
                pBuf[1] = 0xFF;
                firstBuf = FALSE;
            }

            // update expected crc:
			crcExp = SLCD_CalcCrc16( model, pBuf, bufBytes, crcExp );
        }   // ----- end for() loop

        // get a CRC of the chunk in Ext Flash and compare it to what
        // we expect it to be:
        if( ok )
        {
            sprintf( pBuf, "*CEXT 0x%lX 0x%lX", address, address + chunkBytes - 1 );
            ok = SLCD_SendCommandStr( pBuf );
            if( ok )
                ok = SLCD_GetResponseStr( &pResp, SLCD_TYPICAL_TIMEOUT );
            if( ok )
                ok = ( sscanf(pResp, "0x%hx", &crcGot) == 1 );
            if( ok )
                ok = ( crcGot == crcExp );
        }

        // update progress:
        if( ok )
        {
            address += chunkBytes;
            bytesLeft -= chunkBytes;
            bytesDone += chunkBytes;
            percent = ( (100 * bytesDone) / imageSize );
            if( percent != lastPercent )
            {
                printf( "\b\b\b%-3d", percent );
                lastPercent = percent;
            }
        }
    }   // ----- end while() loop

    // end validation progress indication:
    printf( "\r\n" );

    // If all's well, we now write the 1st 2 bytes of external flash with
    // the first 2 bytes of the binary file. This indicates to the firmware that
	// the flash contents are valid:
    if( ok )
    {
		// rewind the file to the beginning:
		rewind( fp );

        // read first 2 bytes from the file, into the buffer:
        ok = ( fread(pBuf, sizeof(char), 2, fp) == bufBytes );

        printf( "Writing Validation code\r\n" );
        ok = SLCD_SendCommandStr( "bdld 4 0 2 2000" );
        if( ok )
            ok = SLCD_GetResponseStr( &pResp, SLCD_TYPICAL_TIMEOUT );
        if( ok )
        {
            ok = SLCD_SendBinaryBuf( pBuf, 2 );
        }
        if( ok )
        {
            ok = SLCD_GetResponseStr( &pResp, SLCD_TYPICAL_TIMEOUT );
            if( ok )
                ok = ( *pResp == '>' );
        }
    }

    // return success/failure:
    return( ok );
}   // ----- end ISP_ValidateExtFlash()


// Upgrade Program Flash from the image in External Flash:
// -- used by SLCDx only, which supports f/w upgrade in .BIN file
// -- returns TRUE if successful, else FALSE.
BOOL ISP_UpgradeProgFlash( void )
{
    BOOL ok = FALSE;
    BOOL programming = FALSE;
    char *pResp, **cpp;

    // send the command to start the upgrade:
    printf( "Executing *upgrade command\r\n" );
    ok = SLCD_SendCommandStr( "*upgrade" );

    // get responses from bootloader and display them until it finishes;
    // the longest period between messages occurs while erasing, which 
    // determines the timeout we use...
    while( ok  )
    {
        ok = SLCD_GetResponseStr( &pResp, ispUPGRADE_ERASE_TIMEOUT );
        if( ok )
        {   
            // if we got an error message, show it and return failure:
            for( cpp = ispUpgradeFailStrings; *cpp; cpp++ )
            {
                if( strstr(pResp, *cpp) )
                {
                    printf( "%s\r\n", pResp );
                    return( FALSE );
                }
            }

            // handle non-error messages:
            if( strstr(pResp, "Programming") )
            {   
                // we just got one of many programming messages; if
                // it's the first one, indicate start of programming:
                if( ! programming )
                {   
                    printf( ":Programming\r\n" );
                    programming = TRUE;
                }
            }
            else
            {   
                // show the message we got:
                printf( "%s\r\n", pResp );
                
                // if the upgrade completed, return success:
                if( strstr(pResp, "Done") )
                    return( TRUE );
            }
        }
    }   // ----- end while() loop

    // return success/failure:
    return( ok );
}   // ----- end ISP_UpgradeProgFlash()


/* -------------------------------------------------------------------------- */
/* PRIVATE CODE                                                               */
/* -------------------------------------------------------------------------- */

// erase a block of external flash 
static BOOL ispEraseExtFlashBlock( SLCD_Model_t model, long blockStartAddrOrSect )
{
    BOOL ok = FALSE;
    char *pCmd, *pResp;

	// get a buffer big enough to hold the max command line:
	pCmd = SLCD_GetCmdBuf();

	// build command string according to SLCD model:
    switch( model )
	{
		case SLCDx:
			// build an xme command:
			sprintf( pCmd, "xme 0x%lX 0xFEEB", blockStartAddrOrSect );
            ok = TRUE;
			break;
		break;

		case SLCD5:
		case SLCD5p:
			// build an xmc command:
			sprintf( pCmd, "xmc 0xFEEB %d", (int)blockStartAddrOrSect );
            ok = TRUE;
		break;
	}

    // send command and evaluate response:
    if( ok )
    {
		ok = SLCD_SendCommandStr( pCmd );

		// 1st response line will be: "Erasing <addr or block number>...\r", with the \r marking
		// the completion of the erase request; this can take awhile: 
		if( ok )
			ok = SLCD_GetResponseStr( &pResp, ispBLOCK_ERASE_TIMEOUT );

		// the next response will start with a '>' if erase succeeded:
		if( ok )
			ok = SLCD_GetResponseStr( &pResp, SLCD_TYPICAL_TIMEOUT );
		if( ok )
			ok = ( *pResp == '>' );
    }

    // return success/failure:
    return( ok );
}   // ----- end ispEraseExtFlashBlock()

/* ----- END OF FILE ----- */
