/*
 *  main.c ..... 'C' source console app's MAIN module.
 */

/* -------------------------------------------------------------------------- */
/* HEADER FILES                                                               */
/* -------------------------------------------------------------------------- */

/* System headers: */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <Windows.h>
#include <sys/types.h>
#include <sys/stat.h>

// additional headers:
#include "commonDefs.h"
#include "serIO.h"
#include "slcd.h"
#include "isp.h"


/* -------------------------------------------------------------------------- */
/* PROTOTYPES FOR PRIVATE CODE                                                */
/* -------------------------------------------------------------------------- */

static void cleanUpAndQuit( int status );
static void showUsageAndQuit( char *progName );
static BOOL validateCmdLine( int argc, char *argv[] );


/* -------------------------------------------------------------------------- */
/* PRIVATE DATA                                                               */
/* -------------------------------------------------------------------------- */

static BOOL l_haveFW = FALSE;
static int l_portNum = 0;
static char *l_fileName = NULL;
static FILE *l_fpImageFile = NULL;
// connect at default rate, unless overridden by the -c command line option:
static long connectBaudrate = 115200;
// transfer at default rate, unless overridden by the -t command line option:
static long txferBaudrate = 115200; 
static char portNameStr[10];					// plenty of room for "COM" + # + NULL
static ISP_SLCDx_TrailerData_t trailerData;		// SLCDx only


/* -------------------------------------------------------------------------- */
/* PROGRAM MAIN()                                                             */
/* -------------------------------------------------------------------------- */

int main( int argc, char *argv[] )
{
    BOOL ok;
    long imageFileSize;
    long flashSize, sectorSize;
    struct _stat imageFileStat;
    int error;
	SLCD_Model_t model;
	SLCD_Version_t version;
	BOOL needExtendedMode;

    // Validate command line and extract args:
    ok = validateCmdLine( argc, argv );
    if( ! ok )
    {
        printf( "ERROR: bad or missing command line arg(s)\r\n" );
        showUsageAndQuit( argv[0] );
    }

    // Open the comm port at the initial connection baud rate:
    sprintf( portNameStr, "COM%d", l_portNum );
    ok = SERIO_Open( portNameStr, connectBaudrate );
    if( ! ok )
    {
        printf( "ERROR: Can't open %s (in use by another program?)\r\n", portNameStr );
        cleanUpAndQuit( -1 );
    }
	
	// Make sure we can connect to the unit:
	ok = SLCD_Connect();
    if( ! ok )
    {
        printf( "ERROR: Can't communicate with SLCD on %s at 115200 baud\r\n", portNameStr );
        cleanUpAndQuit( -1 );
    }

	// If transfer rate is different, switch to it:
	if( txferBaudrate != connectBaudrate )
	{
		ok = SLCD_SetBaudRate( txferBaudrate );
		if( ! ok )
		{
			printf( "ERROR: unable to set SLCD to %ld\r\n", txferBaudrate );
			cleanUpAndQuit( -1 );
		}

		// close the port
		SERIO_Close();

		// reopen at new baud rate
		ok = SERIO_Open( portNameStr, txferBaudrate );
		if( ! ok )
		{
			printf( "ERROR: Can't open %s at %ld baud\r\n", portNameStr, txferBaudrate );
			cleanUpAndQuit( -1 );
		}

		// make sure we can connect to the unit
		ok = SLCD_Connect();
		if( ! ok )
		{
			printf( "ERROR: Can't communicate with SLCD on %s at %ld baud\r\n", portNameStr, txferBaudrate );
			cleanUpAndQuit( -1 );
		}
	}

	// Determine SLCD model and f/w version:
    ok = SLCD_GetModelAndVersion( &model, &version );
    if( ! ok )
    {
        printf( "ERROR: Unable to determine SLCD model\r\n" );
        cleanUpAndQuit( -1 );
    }

	// ignore +fw option if not an SLCDx:
	if( l_haveFW && (model != SLCDx) )
	{
		printf( "NOTE: ignoring +fw option; only applies to SLCDx models\r\n" );
		l_haveFW = FALSE;
	}

    // Open the file for binary read:
    l_fpImageFile = fopen( l_fileName, "rb" );
    ok = ( l_fpImageFile != NULL );
    if( ! ok )
    {
        printf( "ERROR: Can't open file \"%s\"\r\n", l_fileName );
        cleanUpAndQuit( -1 );
    }

    // Get size of image file:
    error = _stat( l_fileName, &imageFileStat );
    if( error )
    {
        printf( "ERROR: Can't get size of file \"%s\"\r\n", l_fileName );
        cleanUpAndQuit( -1 );
    }
    imageFileSize = imageFileStat.st_size;

	// The SLCDx version of BMPLoad puts extra data at the end of the .BIN file
	// for its own use; we need to subtract this from imageFileSize and then
	// make sure the SLCDx is in the appropriate Flash Address Mode (Extended,4M-64K,
	// or Normal, 3M)
	if( model == SLCDx )
	{
		// subtract size of trailer data:
		imageFileSize -= sizeof( trailerData );

		// determine Flash Address Mode needed:
		if( imageFileSize > (3 * 1024 * 1024) )
			needExtendedMode = TRUE;
		else
			needExtendedMode = FALSE;

		// does fw support extended mode?
		if( (version.major >= 2 && version.minor >= 7 && version.sub >= 26) ||
			(version.major >= 2 && version.minor >= 8) )
		{	// it does, so set to the needed mode:
			ok = SLCD_SetFlashExtendMode( needExtendedMode );
			if( ! ok )
			{
				printf( "ERROR: unable to set target SLCD's Flash Address Extension mode\r\n" );
				cleanUpAndQuit( -1 );
			}
		}
		else
		{   // as long as we don't need extended mode, we're ok:
			ok = ( ! needExtendedMode );
			if( ! ok )
			{
				printf( "ERROR: unable to set target SLCD's Flash Address Extension mode\r\n" );
				cleanUpAndQuit( -1 );
			}
		}
	}

	// Make sure we have a valid .BIN file:
	ok = ISP_ValidateBinFile( model, l_fpImageFile, imageFileSize );
	if( ! ok )
	{
		printf( "ERROR: file data format incorrect for SLCD model\r\n" );
        cleanUpAndQuit( -1 );
	}

    // Get size of External Flash:
    ok = ISP_GetExtFlashSize( model, &flashSize, &sectorSize );
    if( ! ok )
    {
        printf( "ERROR: Can't get size of External Flash\r\n" );
        cleanUpAndQuit( -1 );
    }

	// Make sure Flash is big enough:
    ok = ( imageFileSize <= flashSize );
    if( ! ok )
    {
        printf( "ERROR: Flash too small; have %ld bytes, need %ld\r\n", flashSize, imageFileSize );
        cleanUpAndQuit( -1 );
    }

    // Erase SLCD External Flash:
    ok = ISP_EraseExtFlash( model, imageFileSize, sectorSize );
    if( ! ok )
    {
        printf( "ERROR: External Flash failed to erase\r\n" );
        cleanUpAndQuit( -1 );
    }

    // Write image file into SLCD External Flash:
    ok = ISP_WriteFileToExtFlash( model, l_fpImageFile, imageFileSize );
    if( ! ok )
    {
        printf( "ERROR: External Flash failed to program\r\n" );
        cleanUpAndQuit( -1 );
    }

    // Validate External Flash:
    ok = ISP_ValidateExtFlash( model, l_fpImageFile, imageFileSize );
    if( ! ok )
    {
        printf( "ERROR: External Flash failed validation\r\n" );
        cleanUpAndQuit( -1 );
    }

    // If there's f/w included, upgrade Program Flash from the
    // image now in External Flash:
    if( l_haveFW )
    {
        ok = ISP_UpgradeProgFlash();
        if( ! ok )
        {
            printf( "ERROR: Flash upgrade command failed\r\n" );
            cleanUpAndQuit( -1 );
        }
    }

	// if model is not SLCDx, issue *LOAD command to load validated image
	// from Flash to RAM
	if( model != SLCDx  )
	{
		ok = SLCD_Load();
		if( ! ok )
		{
			printf( "ERROR: Can't load SLCD5/SLCD5+ RAM from Flash\r\n" );
			cleanUpAndQuit( -1 );
		}
	}

    // If we get here, we succeeded!  Clean up and return:
    printf( "SUCCESS !!!\r\n" );
    cleanUpAndQuit( 0 );
}   // ----- end main()


/* -------------------------------------------------------------------------- */
/* PRIVATE CODE                                                               */
/* -------------------------------------------------------------------------- */

// Clean up and exit with the given status:
static void cleanUpAndQuit( int status )
{
	BOOL ok;

	if( l_fpImageFile )
        fclose( l_fpImageFile );
 
	// don't know status of communication, so check the connection
	ok = SLCD_Check();
	if( ! ok )
	{
		printf( "Error: Can't communicate with SLCD.\r\n" );
	}

	// revert to connect baud rate?
	if( ok && (txferBaudrate != connectBaudrate) )
	{
		ok = SLCD_SetBaudRate( connectBaudrate );
		if( ! ok )
		{
			printf( "ERROR: unable to reset SLCD to %ld baud\r\n", connectBaudrate );
		}
		// close the port
		SERIO_Close();
		// reopen at new baud rate
		ok = SERIO_Open( portNameStr, connectBaudrate );
		if( ! ok )
		{
			printf( "ERROR: Can't reopen %s at %d baud\r\n", portNameStr, connectBaudrate );
		}
		// make sure we can connect to the unit
		ok = SLCD_Connect();
		if( ! ok )
		{
			printf( "ERROR: Can't communicate with SLCD on %s at %d baud\r\n", portNameStr, connectBaudrate );
		}
	}
	
	SERIO_Close();
    exit( status );
}   // ----- end cleanUpAndQuit()


// Show a usage message and quit:
static void showUsageAndQuit( char *progName )
{
	printf( "Version: %s\r\n", __DATE__ );
    printf( "USAGE: %s <commPortNum> [+fw] [-c<baudrate>] [-t<baudrate>] <fileName>\r\n", progName );
    printf( " -- <commPortNum> should be 1-9\r\n" );
    printf( " -- [+fw] optional, indicates firmware is included\r\n" );
	printf( " -- [-c<baudrate>] optional, baud rate for connecting to SLCD\r\n" );
	printf( "    (default is 115200 baud)\r\n" );
	printf( " -- [t<baudrate>] optional, baud rate for transferring file to SLCD\r\n" );
	printf( "    (default is connecting baud rate)\r\n" );
    printf( " -- <fileName> is the path of a .bin (or .fwu) file built by BMPLOAD.EXE\r\n" );
    exit( 1 );
}   // ----- end showUsageAndQuit()


// Validate cmdline and extract args:
static BOOL validateCmdLine( int argc, char *argv[] )
{
    BOOL ok;
	BOOL gotTransferRate = FALSE;
    int iTmp;
    char *cp;
	int argIdx;

    //  must have cmdname + 2 or more args:
    ok = ( argc >= 3 );
	
	argc-=2;	// account for name and com port arg

    // 1st arg must be COM port number, [1-9]:
    if( ok )
    {
        iTmp = atoi( argv[1] );
        ok = ( (iTmp > 0) && (iTmp < 10) );
        if( ok )
            l_portNum = iTmp;
    }

    // next arg is optional, will tell us if there's fw or not:
	// this option is available for the SLCDx only
    argIdx = 2;
	if( ok && argc )
    {
        cp = argv[argIdx];
        if( (strcmp("+fw", cp) == 0) || (strcmp("+FW", cp) == 0) )
		{
            l_haveFW = TRUE;
			printf( "Firmware update option selected\r\n");
			argIdx++;
			argc--;
		}
        else
		{
            l_haveFW = FALSE;
		}
    }

	// next arg is optional, will tell us the baudrate we should use to
	// connect (SLCD could power up with non-default baudrate)
	if( ok && argc )
    {
        if( 1 == sscanf( argv[argIdx], "-c%ld", &connectBaudrate) )
		{
			printf( "Connection baudrate: %ld\r\n", connectBaudrate );
			argIdx++;
			argc--;
		}
    }

	// next arg is optional, will tell us if we are to use a different baudrate
	// for the transfer.
	if( ok && argc )
    {
        if( 1 == sscanf( argv[argIdx], "-t%ld", &txferBaudrate) )
		{
			gotTransferRate = TRUE;
			printf( "Transfer baudrate: %ld\r\n", txferBaudrate );
			argIdx++;
			argc--;
		}
    }

    // last arg must be a ".bin" or ".fwu" fileName:
    if( ok && argc )
    {
        iTmp = (int)strlen( argv[argIdx] );
        ok = ( iTmp > 4 );
        if( ok )
        {
            cp = ( argv[argIdx] + iTmp - 4 );
            ok = ( 
                    (strcmp(".bin", cp) == 0) || (strcmp(".BIN", cp) == 0) ||
                    (strcmp(".fwu", cp) == 0) || (strcmp(".FWU", cp) == 0)
                 );
        }
        if( ok )
            l_fileName = argv[argIdx];
    }

	// if not given a transfer rate, use the connecting rate:
	if( ok && (! gotTransferRate) )
		txferBaudrate = connectBaudrate;

    // return success/failure:
    return( ok );
}   // ----- end validateCmdLine()

/* ----- END OF FILE ----- */
