/*
 *  serIO.c ..... 'C' source for the SERIAL I/O module.
 */


/* -------------------------------------------------------------------------- */
/* HEADER FILES                                                               */
/* -------------------------------------------------------------------------- */

/* The header that defines the Public interface to this module: */
#define DEF_SERIO_GLOBALS
#include "serIO.h"
#undef DEF_SERIO_GLOBALS

/* Additional headers required by this file: */
#include <windows.h>


/* -------------------------------------------------------------------------- */
/* PRIVATE DATA                                                               */
/* -------------------------------------------------------------------------- */

// handle for the serial port:
static HANDLE serIOPortHandle = INVALID_HANDLE_VALUE;


/* -------------------------------------------------------------------------- */
/* PUBLIC CODE                                                                */
/* -------------------------------------------------------------------------- */
// (public types, constants, data, & prototypes in serIO.h)

// open the comm port:
// -- read timeout is set for 10 milliseconds
// -- write timeout is disabled
// -- port is set for 115200 baud, 8N1, Xon/Xoff TX flow ctl, binary xfer mode
// -- returns TRUE if successful
BOOL SERIO_Open( char *comPortName, int baud )
{
    BOOL ok = TRUE;
    COMMTIMEOUTS cto = { 0, 0, SERIO_READ_TIMEOUT_MSEC, 0, 0 };
    DCB dcb;

    // make sure it's already closed:
    SERIO_Close();

    // try to get a handle:
    serIOPortHandle = CreateFile( comPortName,
        GENERIC_READ | GENERIC_WRITE, 
        0,                  // share mode
        NULL,               // address of security descriptor
        OPEN_EXISTING,      // how to create
        0,                  // file attributes
        NULL                // handle of file with attributes to copy
        );
    ok = ( serIOPortHandle != INVALID_HANDLE_VALUE );

    // setup timeouts:
    if( ok )
        ok = SetCommTimeouts( serIOPortHandle, &cto );

    // set for 115200, 8N1, with outbound Xon/Xoff flow control:
    if( ok )
    {
        // clear the dcb:
        memset( &dcb, 0, sizeof(dcb) );
        dcb.DCBlength = sizeof( dcb );
        // windows only supports binary mode xfers:
        dcb.fBinary = TRUE;
        // set the stuff we want to use:
        dcb.BaudRate = baud;
        dcb.ByteSize = 8;
        dcb.Parity = NOPARITY;
        dcb.StopBits = ONESTOPBIT;

        // want Xon/Xoff flow control for TX:
        dcb.XonChar = 0x11;
        dcb.XoffChar = 0x13;
        dcb.fOutX = TRUE;

        // set the comm state:
        ok = SetCommState( serIOPortHandle, &dcb );
    }

    // close if there was an error:
    if( ! ok )
        SERIO_Close();

    // return result:
    return( ok );
}   // ----- end SERIO_Open()


// close the comm port:
void SERIO_Close( void )
{
    if( serIOPortHandle != INVALID_HANDLE_VALUE )
    {
        CloseHandle( serIOPortHandle );
        serIOPortHandle = INVALID_HANDLE_VALUE;
    }
}   // ----- end SERIO_Close()


// get a char from the serial port (or timeout):
// -- returns -1 if no char received
int SERIO_GetChar( void )
{
    char ch;
    int retVal = -1;
    DWORD bytesRead = 0;

    if( serIOPortHandle != INVALID_HANDLE_VALUE )
    {
        if( ReadFile(serIOPortHandle, &ch, 1, &bytesRead, NULL) )
            if( bytesRead )
                retVal = ch;
    }
    return( retVal );
}   // ----- end SERIO_GetChar()


// send a char to the serial port:
void SERIO_PutChar( char ch )
{
    DWORD bytesSent;

    if( serIOPortHandle != INVALID_HANDLE_VALUE )
    {
        WriteFile( serIOPortHandle, &ch, 1, &bytesSent, NULL );
        FlushFileBuffers( serIOPortHandle );
    }
}   // ----- end SERIO_PutChar()


// send a string to the serial port:
void SERIO_PutString( char *str )
{
    DWORD bytesSent;

    if( str && *str && (serIOPortHandle != INVALID_HANDLE_VALUE) )
    {
        while( *str )
            WriteFile( serIOPortHandle, str++, 1, &bytesSent, NULL );
        FlushFileBuffers( serIOPortHandle );
    }
}   // ----- end SERIO_PutString()


// send the contents of a binary buffer to the serial port:
void SERIO_SendBytes( char *buf, int len )
{
    DWORD bytesSent;

    if( buf && (len > 0) )
    {
        WriteFile( serIOPortHandle, buf, len, &bytesSent, NULL );
        FlushFileBuffers( serIOPortHandle );
    }
}   // ----- end SERIO_SendBytes()


// flush the input buffer for the requested amount of time:
void SERIO_FlushInput( int mSec )
{
    if( serIOPortHandle != INVALID_HANDLE_VALUE )
    {
        while( mSec < 250 )
            if( SERIO_GetChar() < 0 )
                mSec += SERIO_READ_TIMEOUT_MSEC;
    }
}   // ----- end SERIO_FlushInput()


/* ----- END OF FILE ----- */
