/*
 *  slcd.h ..... 'C' header for the SLCD module.
 */

/* prevent multiple inclusion of this file: */
#ifndef _SLCD_H
#define _SLCD_H


/* -------------------------------------------------------------------------- */
/* HEADER FILES                                                               */
/* -------------------------------------------------------------------------- */

/* Additional headers required by this file: */
#include "commonDefs.h"


/* -------------------------------------------------------------------------- */
/* PUBLIC TYPES                                                               */
/* -------------------------------------------------------------------------- */

// SLCD Controller models
typedef enum
{
    UNKNOWN = 0,
    SLCDx,			// SLCD43, SLCD6, SLCD+
    SLCD5,
	SLCD5p			// SLCD5+
} SLCD_Model_t;

typedef struct
{
	int major;
	int minor;
	int sub;
} SLCD_Version_t;


/* -------------------------------------------------------------------------- */
/* PUBLIC CONSTANTS                                                           */
/* -------------------------------------------------------------------------- */

// Max SLCD xfer, in bytes:
#define SLCD_MAXBYTES           (128)

// Typical SLCD response timeout, in mSec:
#define SLCD_TYPICAL_TIMEOUT    (1000)

// SLCD "load" timeout, in mSec:
#define SLCD_LOAD_TIMEOUT		(10000)

// Special value, causes minimum possible timeout to be used:
#define SLCD_MINIMUM_TIMEOUT    (-1)

// Seed value for 16-Bit CRC:
#define SLCD_SEED16             0xFFFF


/* -------------------------------------------------------------------------- */
/* PROTOTYPES FOR PUBLIC CODE                                                 */
/* -------------------------------------------------------------------------- */

// Connect to the SLCD control port:
// -- tries to establish a connection to the SLCD, or times out
//    after 1 second
// -- returns TRUE if successful, else FALSE.
BOOL SLCD_Connect( void );

// Validate connection to the SLCD control port:
// -- send <cr>, look for '>' followed by <cr>
// -- returns TRUE if successful, else FALSE.
BOOL SLCD_Check( void );

// Set baud rate on the SLCD control port:
// -- returns TRUE if successful, else FALSE.
BOOL SLCD_SetBaudRate( long baud );

// Returns a pointer to a buffer big enough to hold the max length command line:
char *SLCD_GetCmdBuf( void );

// Send a command string to the SLCD:
// -- flushes the serial input buffer before sending the command
// -- blocking, returns after the last char has been sent out the
//    serial port
// -- returns TRUE if successful, else FALSE.
BOOL SLCD_SendCommandStr( char *cmdStr );

// Get a response string from the SLCD:
// -- returns TRUE if a response is received withing the timeout
//    AND passes back a pointer to the buffer containing the response
//    string
// -- else, returns FALSE
BOOL SLCD_GetResponseStr( char **pRespStr, int timeoutMsec );

// Send the contents of a binary buffer to the SLCD:
// -- blocking, returns after the last byte has been sent out the
//    serial port
// -- returns TRUE if successful, else FALSE.
BOOL SLCD_SendBinaryBuf( char *buf, int len );

// Generate a 16-Bit CRC of the given buffer, using the given seed and
// the algorithm for the given model:
unsigned short SLCD_CalcCrc16( SLCD_Model_t model, char *buf, int len, unsigned short seed );

// Get the version string from the SLCD and pass back the model and f/w version:
// -- returns TRUE if successful, else FALSE.
BOOL SLCD_GetModelAndVersion( SLCD_Model_t *pModel, SLCD_Version_t *pVers );

// Tell the SLCD to reset itself:
// -- returns TRUE if successful, else FALSE.
BOOL SLCD_Reset( void );

// Tell an SLCDx to set its Flash Address Extend Mode:
// -- returns TRUE if successful, else FALSE.
BOOL SLCD_SetFlashExtendMode( BOOL fmxON );

// Tell an SLCD5/SLCD5+ to load binary image:
// -- returns TRUE if successful, else FALSE.
BOOL SLCD_Load( void );

/* end of multiple inclusion protection: */
#endif  /* not _SLCD_H */

/* ----- END OF FILE ----- */
